/*
   SynScopeV

   Copyright (C) 2008,2009:
         Daniel Roggen, droggen@gmail.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QtGui/QMainWindow>
#include <QLabel>
#include <QDir>
#include <QSplitter>
#include <QScrollArea>

#include "ui_mainwindow.h"

#include "dscopeqtextwidget.h"
#include "referencesdialog.h"
#include "exportvideo.h"

#include "data.h"

namespace Ui
{
    class MainWindow;
}

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    MainWindow(QWidget *parent = 0);
    ~MainWindow();

private:
   Ui::MainWindow *ui;
   ReferencesDialog *RefDialog;

   std::vector<SIGNALDATA> SignalData,SignalDataUndo; // SignalData[source]

   // Signal/video splitter & scroll areas
   QSplitter *signalVideoSplitter;
   QScrollArea *scrollArea,*videoScrollArea;

   // Signal related
   VIEWDATAS ViewData;
   QFrame *mainFrame;                 // Frame within the scroll area
   QLayout *signalVerticalLayout;     // Vertical layouting within the frame within the scroll area

   // Video related
   VIDEOVIEWDATAS VideoViewData;
   QFrame *mainVideoFrame;
   QHBoxLayout *videoLayout;

   bool undoable;
   REFERENCEPOINTS RefPoints,RefPointsUndo;          // Reference points between pairs of signals
   RELATIONS RelationsAvailable;       // Relations between pairs of signals (i.e. transfer formula)
   RELATIONS RelationsActive;          // Active relations, i.e. subset of the possible relations. The recursion data are not used in this structure.
   bool LinkSignals;                   // General flag indicating whether signals should be linked.
   bool DeafToSliderEvents;            //
   int DeafToVideoSliderEvents;        // Handles the asynchronous notification of slider updates

   QLabel *SB_LabelSource,*SB_LabelPosition,*SB_LabelCenter,*SB_LabelSignal;
   string CurrentLayoutString;


   int source_primary,source_secondary;    // Primary and secondary sources (-1 for not assigned).

   QDir path;


   int timer;

   void AddSignalView(VIEWDATA &vd);
   void DeleteSignalViews();
   void SetAllSliderPosition(bool all=true,vector<int> vid=vector<int>(0),vector<int> sid=vector<int>(0));
   void PreparePlotView(int view,vector<vector<int> *> &vd,vector<unsigned> &vc);
   void PlotView(int signal);
   int ParseLayoutString(string str,VIEWDATAS &vdv);
   int ParseSourceString(string str,SOURCEFILES &src);

   std::vector<std::vector<int> > LoadSignalFile(string file,int &ok,bool report=true);

   void on_Source_Slided(int sourceidx,int value);
   void AdjustSignalViewSlider();
   void SourceClick(int source,Qt::MouseButton button,int sourcex);
   void VisualizePrimarySecondary();

   bool OLSQRegress(std::vector<double> x1,std::vector<double> x2, double &a, double &b);
   bool OrthogonalLSQRegression(std::vector<double> x1,std::vector<double> x2, double &a, double &b);
   //bool IsRelationConflicting(RELATIONS &Relations, int x1,int x2);

   void loadConfiguration(QString fileName);

   map<int,RESAMPLEREL> ResampleLogic(vector<RELATIONS> Graphs,bool prepend);
   void ResampleInPlace(map<int,RESAMPLEREL> m,bool prepend,bool postpend);
   void ResampleMatlab(map<int,RESAMPLEREL> m,bool nn,bool prepend,bool postpend);
   SIGNALDATA ResampleSignalData(SIGNALDATA &sigdataorg,double resample,int padleft,int padright);
   void Resample(bool inplace);

   void SaveSignal(vector<vector<int> > &source,QString strtitle);

   void ComputeRelations();
   void UpdateRelationsActive();
   void SetSBLabelPointer(int idx);
   void MergeFiles();

   void SetupViews(string layout);
   void SetUndoable(bool u);
   void ClearActiveRelations();
   REFERENCEPOINTS PruneRefPoints(REFERENCEPOINTS v,QString &str,bool display=true);
   void SetReferencePoints(REFERENCEPOINTS ref);

   void addVideo(QString file);
   void DeleteVideoViews();
   unsigned GetCenterSample(int idx,bool effective=true);
   void SetCenterSample(int idx,int centersample);
   bool IsIdxVideo(int idx);
   bool IsIdxSignal(int idx);
   int VID2IDX(int vid);
   int SID2IDX(int sid);
   int IDX2SID(int idx);
   int IDX2VID(int idx);
   int GetNumSources();
   int GetNumSignalViews();
   int GetNumVideoViews();
   void PlotAllSignalViews();

   int GetPrimarySource();

   // Video

   bool GetVideoParameters(unsigned &videow,unsigned &videoh,unsigned &bitrate,unsigned &gop,unsigned &signalstep,bool &drawtitle,bool &dbl);
   int SaveSignalToVideo(unsigned source,unsigned videow,unsigned videoh,unsigned bitrate,unsigned gop,unsigned signalstep,bool drawtitle,bool dbl,QString videoFileName,QString progressTitle);

   void findlinks();



   void image2Pixmap(QImage &img,QPixmap &pixmap);


protected:
    //virtual void timerEvent(QTimerEvent *event);

private slots:
    //void on_pushButtonApply_clicked();
    //void on_pushButton_2_clicked();
    //void on_pushButton_clicked();
    //void on_toolButton_clicked();
    /*void on_actionTest3_triggered();
    void on_actionTest2_triggered();
    void on_actionTest1_triggered();*/
    void on_actionMosaic_video_triggered();
    void on_actionVideo_of_all_signal_sources_triggered();
    void on_actionVideo_from_primary_source_triggered();
    void on_action_Quit_triggered();
    void on_actionSet_working_directory_triggered();
    void on_actionClear_reference_points_triggered();
    void on_actionAdd_reference_point_triggered();
    void on_actionShow_reference_points_and_links_triggered();
    void on_actionMerge_linked_files_triggered();
    void on_actionHow_to_triggered();
    void on_actionAbout_triggered();
    void on_actionUndo_triggered();
    void on_actionSave_primary_source_triggered();
    void on_actionExport_Matlab_resample_code_triggered();
    void on_action_Resample_triggered();
    void on_actionAbout_Q_triggered();
    void on_actionShow_available_sources_triggered();
    void on_actionLoad_reference_points_triggered();
    void on_actionSave_reference_points_triggered();
    void on_actionMatrix_triggered();
    void on_actionText_triggered();
    void on_actionSave_configuration_triggered();
    void on_actionLoad_configuration_triggered();
    //void on_pushButton_ShowVideo_clicked();
    //void on_pushButton_ComputeRelations_clicked();
    //void on_pushButton_3_clicked();
    void on_pushButton_LoadSource_clicked();
    void on_pushButton_ApplyView_clicked();
    //void on_pushButtonPlay_clicked();

    void on_Slider_valueChanged(int value);
    void on_Source_mousePressed(Qt::MouseButton button, int samplex);
    void on_Source_mouseMoved(int samplex);

    void on_AddReference();
    void on_RemoveReference(int idx);
    void on_ReferenceSelected(int idx);

    void on_ChangedActiveRelations();

    void on_Video_tick ( qint64 time );
    void on_videoPlayPauseButton(bool);
    void on_Repaint();

    void stateChanged(Phonon::State newState, Phonon::State  oldState);
};




#endif // MAINWINDOW_H

